import { AuthService } from '@/services/auth.service';
import { RequestHandler } from 'express';
import { plainToInstance } from 'class-transformer';
import { RegisterUserDto } from '@/dto/register-user.dto';
import { SignInUserDto } from '@/dto/sign-in-user.dto';

export class AuthController {
  private service: AuthService;

  constructor() {
    this.service = new AuthService();
  }

  signIn: RequestHandler = async (req, res): Promise<void> => {
    try {
      const signInUserDto = plainToInstance(SignInUserDto, req.body);
      const user = await this.service.signIn(signInUserDto);
      res.send(user);
    } catch (e) {
      res.send((e as Error).message);
    }
  };

  register: RequestHandler = async (req, res): Promise<void> => {
    try {
      const registerUserDto = plainToInstance(RegisterUserDto, req.body);
      const user = await this.service.register(registerUserDto);
      res.send(user);
    } catch (e) {
      if ((e as { code: string }).code === 'ER_DUP_ENTRY') {
        res.send({ error: { message: 'User already exists' } });
      } else {
        res.status(500).send({ error: { message: 'Oops something went wrong' } });
      }
    }
  };

  secret: RequestHandler = async (req, res) => {
    try {
      const token = req.header('Authorization');
      if (!token) return res.status(401).send({ error: { message: 'No token present' } });

      const user = await this.service.getUserByToken(token);
      if (!user) return res.status(401).send({ error: { message: 'Wrong token' } });

      return res.send({ message: 'some secret message' });
    } catch (e) {
      return res.status(500).send({ error: { message: 'Internal server error' } });
    }
  };
}
